@@ -5,13 +5,21 @@ use std::ffi::OsStr;
 use std::ffi::OsString;
 use std::mem;
 use std::ops::Deref;
+use std::ops::Index;
+use std::ops::Range;
+use std::ops::RangeFrom;
+use std::ops::RangeFull;
+use std::ops::RangeInclusive;
+use std::ops::RangeTo;
+use std::ops::RangeToInclusive;
 use std::str;
 
 use super::imp::raw;
-use super::pattern::Encoder;
-use super::pattern::Pattern;
+use super::iter::Split;
+use super::pattern::Encoded as EncodedPattern;
 use super::OsStrBytes;
 use super::OsStringBytes;
+use super::Pattern;
 
 #[cfg(feature = "print_bytes")]
 use print_bytes::Bytes;
@@ -41,8 +49,8 @@ fn rfind_pattern(string: &[u8], pat: &[u8]) -> Option<usize> {
 
 macro_rules! impl_trim_matches {
     ( $self:ident , $pat:expr , $strip_method:ident ) => {{
-        let mut encoder = $pat.__into_encoder();
-        let pat = encoder.__encode();
+        let pat = $pat.__encode();
+        let pat = pat.__get();
         if pat.is_empty() {
             return $self;
         }
@@ -54,24 +62,21 @@ macro_rules! impl_trim_matches {
         while let Some(substring) = string.$strip_method(pat) {
             string = substring;
         }
-        unsafe { Self::from_raw_bytes_unchecked(string) }
+        Self::from_raw_bytes_unchecked(string)
     }};
 }
 
-macro_rules! impl_split_once {
-    ( $self:ident , $pat:expr , $find_fn:ident ) => {{
-        let mut encoder = $pat.__into_encoder();
-        let pat = encoder.__encode();
+macro_rules! impl_split_once_raw {
+    ( $self:ident , $pat:expr , $find_fn:expr ) => {{
+        let pat = $pat.__get();
 
         let index = $find_fn(&$self.0, pat)?;
         let prefix = &$self.0[..index];
         let suffix = &$self.0[index + pat.len()..];
-        unsafe {
-            Some((
-                Self::from_raw_bytes_unchecked(prefix),
-                Self::from_raw_bytes_unchecked(suffix),
-            ))
-        }
+        Some((
+            Self::from_raw_bytes_unchecked(prefix),
+            Self::from_raw_bytes_unchecked(suffix),
+        ))
     }};
 }
 
@@ -85,6 +90,19 @@ macro_rules! impl_split_once {
 /// representation is not stable. Transmuting between this type and any other
 /// causes immediate undefined behavior.
 ///
+/// # Indices
+///
+/// Methods of this struct that accept indices require that the index lie on a
+/// UTF-8 boundary. Although it is possible to manipulate platform strings
+/// based on other indices, this crate currently does not support them for
+/// slicing methods. They would add significant complication to the
+/// implementation and are generally not necessary. However, all indices
+/// returned by this struct can be used for slicing.
+///
+/// On Unix, all indices are permitted, to avoid false positives. However,
+/// relying on this implementation detail is discouraged. Platform-specific
+/// indices are error-prone.
+///
 /// [unspecified encoding]: super#encoding
 #[derive(Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
 #[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "raw_os_str")))]
@@ -92,9 +110,9 @@ macro_rules! impl_split_once {
 pub struct RawOsStr([u8]);
 
 impl RawOsStr {
-    unsafe fn from_raw_bytes_unchecked(string: &[u8]) -> &Self {
+    fn from_raw_bytes_unchecked(string: &[u8]) -> &Self {
         // SAFETY: This struct has a layout that makes this operation safe.
-        mem::transmute(string)
+        unsafe { mem::transmute(string) }
     }
 
     /// Converts a platform-native string into a representation that can be
@@ -122,9 +140,9 @@ impl RawOsStr {
     #[must_use]
     pub fn new(string: &OsStr) -> Cow<'_, Self> {
         match string.to_raw_bytes() {
-            Cow::Borrowed(string) => unsafe {
+            Cow::Borrowed(string) => {
                 Cow::Borrowed(Self::from_raw_bytes_unchecked(string))
-            },
+            }
             Cow::Owned(string) => Cow::Owned(RawOsString(string)),
         }
     }
@@ -149,8 +167,7 @@ impl RawOsStr {
     #[inline]
     #[must_use]
     pub fn from_str(string: &str) -> &Self {
-        let string = string.as_bytes();
-        unsafe { Self::from_raw_bytes_unchecked(string) }
+        Self::from_raw_bytes_unchecked(string.as_bytes())
     }
 
     /// Returns the byte string stored by this container.
@@ -224,8 +241,8 @@ impl RawOsStr {
     where
         P: Pattern,
     {
-        let mut encoder = pat.__into_encoder();
-        let pat = encoder.__encode();
+        let pat = pat.__encode();
+        let pat = pat.__get();
 
         self.0.ends_with(pat)
     }
@@ -272,8 +289,8 @@ impl RawOsStr {
     where
         P: Pattern,
     {
-        let mut encoder = pat.__into_encoder();
-        let pat = encoder.__encode();
+        let pat = pat.__encode();
+        let pat = pat.__get();
 
         find_pattern(&self.0, pat)
     }
@@ -311,8 +328,8 @@ impl RawOsStr {
     /// ```
     /// use os_str_bytes::RawOsStr;
     ///
-    /// let raw = RawOsStr::from_str("foobar");
-    /// assert_eq!(6, raw.raw_len());
+    /// assert_eq!(6, RawOsStr::from_str("foobar").raw_len());
+    /// assert_eq!(0, RawOsStr::from_str("").raw_len());
     /// ```
     #[inline]
     #[must_use]
@@ -341,12 +358,19 @@ impl RawOsStr {
     where
         P: Pattern,
     {
-        let mut encoder = pat.__into_encoder();
-        let pat = encoder.__encode();
+        let pat = pat.__encode();
+        let pat = pat.__get();
 
         rfind_pattern(&self.0, pat)
     }
 
+    pub(super) fn rsplit_once_raw<P>(&self, pat: &P) -> Option<(&Self, &Self)>
+    where
+        P: EncodedPattern,
+    {
+        impl_split_once_raw!(self, pat, rfind_pattern)
+    }
+
     /// Equivalent to [`str::rsplit_once`].
     ///
     /// # Panics
@@ -365,12 +389,107 @@ impl RawOsStr {
     /// );
     /// assert_eq!(None, raw.rsplit_once("of"));
     /// ```
+    #[inline]
     #[must_use]
     pub fn rsplit_once<P>(&self, pat: P) -> Option<(&Self, &Self)>
     where
         P: Pattern,
     {
-        impl_split_once!(self, pat, rfind_pattern)
+        self.rsplit_once_raw(&pat.__encode())
+    }
+
+    // https://github.com/rust-lang/rust/blob/49c68bd53f90e375bfb3cbba8c1c67a9e0adb9c0/src/libcore/str/mod.rs#L2184-L2221
+    #[cold]
+    #[inline(never)]
+    #[track_caller]
+    fn index_boundary_error(&self, index: usize) -> ! {
+        debug_assert!(raw::is_continuation(self.0[index]));
+
+        let start = self.0[..index]
+            .iter()
+            .rposition(|&x| !raw::is_continuation(x))
+            .expect("invalid raw bytes");
+        let mut end = index + 1;
+        end += self.0[end..]
+            .iter()
+            .position(|&x| !raw::is_continuation(x))
+            .unwrap_or_else(|| self.raw_len() - end);
+        let code_point = raw::decode_code_point(&self.0[start..end]);
+        panic!(
+            "byte index {} is not a valid boundary; it is inside U+{:04X} \
+            (bytes {}..{})",
+            index, code_point, start, end,
+        );
+    }
+
+    #[track_caller]
+    fn check_bound(&self, index: usize) {
+        if let Some(&byte) = self.0.get(index) {
+            if raw::is_continuation(byte) {
+                self.index_boundary_error(index);
+            }
+        }
+    }
+
+    /// Equivalent to [`str::split`], but empty patterns are not accepted.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the pattern is a byte outside of the ASCII range or empty.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use os_str_bytes::RawOsStr;
+    ///
+    /// let raw = RawOsStr::from_str("foobar");
+    /// assert_eq!(["f", "", "bar"], *raw.split("o").collect::<Vec<_>>());
+    /// ```
+    #[inline]
+    #[must_use]
+    pub fn split<P>(&self, pat: P) -> Split<'_, P>
+    where
+        P: Pattern,
+    {
+        Split::new(self, pat)
+    }
+
+    /// Equivalent to [`str::split_at`].
+    ///
+    /// # Panics
+    ///
+    /// Panics if the index is not a [valid boundary].
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use os_str_bytes::RawOsStr;
+    ///
+    /// let raw = RawOsStr::from_str("foobar");
+    /// assert_eq!(
+    ///     ((RawOsStr::from_str("fo"), RawOsStr::from_str("obar"))),
+    ///     raw.split_at(2),
+    /// );
+    /// ```
+    ///
+    /// [valid boundary]: #indices
+    #[inline]
+    #[must_use]
+    pub fn split_at(&self, mid: usize) -> (&Self, &Self) {
+        self.check_bound(mid);
+
+        let (prefix, suffix) = self.0.split_at(mid);
+        (
+            Self::from_raw_bytes_unchecked(prefix),
+            Self::from_raw_bytes_unchecked(suffix),
+        )
+    }
+
+    pub(super) fn split_once_raw<P>(&self, pat: &P) -> Option<(&Self, &Self)>
+    where
+        P: EncodedPattern,
+    {
+        impl_split_once_raw!(self, pat, find_pattern)
     }
 
     /// Equivalent to [`str::split_once`].
@@ -391,12 +510,13 @@ impl RawOsStr {
     /// );
     /// assert_eq!(None, raw.split_once("of"));
     /// ```
+    #[inline]
     #[must_use]
     pub fn split_once<P>(&self, pat: P) -> Option<(&Self, &Self)>
     where
         P: Pattern,
     {
-        impl_split_once!(self, pat, find_pattern)
+        self.split_once_raw(&pat.__encode())
     }
 
     /// Equivalent to [`str::starts_with`].
@@ -420,8 +540,8 @@ impl RawOsStr {
     where
         P: Pattern,
     {
-        let mut encoder = pat.__into_encoder();
-        let pat = encoder.__encode();
+        let pat = pat.__encode();
+        let pat = pat.__get();
 
         self.0.starts_with(pat)
     }
@@ -472,12 +592,10 @@ impl RawOsStr {
     where
         P: Pattern,
     {
-        let mut encoder = pat.__into_encoder();
-        let pat = encoder.__encode();
+        let pat = pat.__encode();
+        let pat = pat.__get();
 
-        self.0
-            .strip_prefix(pat)
-            .map(|x| unsafe { Self::from_raw_bytes_unchecked(x) })
+        self.0.strip_prefix(pat).map(Self::from_raw_bytes_unchecked)
     }
 
     /// Equivalent to [`str::strip_suffix`].
@@ -504,12 +622,10 @@ impl RawOsStr {
     where
         P: Pattern,
     {
-        let mut encoder = pat.__into_encoder();
-        let pat = encoder.__encode();
+        let pat = pat.__encode();
+        let pat = pat.__get();
 
-        self.0
-            .strip_suffix(pat)
-            .map(|x| unsafe { Self::from_raw_bytes_unchecked(x) })
+        self.0.strip_suffix(pat).map(Self::from_raw_bytes_unchecked)
     }
 
     /// Converts this representation back to a platform-native string.
@@ -648,6 +764,13 @@ impl AsRef<RawOsStr> for String {
     }
 }
 
+impl Default for &RawOsStr {
+    #[inline]
+    fn default() -> Self {
+        RawOsStr::from_str("")
+    }
+}
+
 impl<'a> From<&'a RawOsStr> for Cow<'a, RawOsStr> {
     #[inline]
     fn from(other: &'a RawOsStr) -> Self {
@@ -655,6 +778,36 @@ impl<'a> From<&'a RawOsStr> for Cow<'a, RawOsStr> {
     }
 }
 
+macro_rules! r#impl {
+    (
+        $index_type:ty
+        $(, $index_var:ident , $first_bound:expr $(, $second_bound:expr)?)?
+    ) => {
+        impl Index<$index_type> for RawOsStr {
+            type Output = Self;
+
+            #[inline]
+            fn index(&self, idx: $index_type) -> &Self::Output {
+                $(
+                    let $index_var = &idx;
+                    self.check_bound($first_bound);
+                    $(self.check_bound($second_bound);)?
+                )?
+
+                Self::from_raw_bytes_unchecked(&self.0[idx])
+            }
+        }
+    };
+}
+r#impl!(Range<usize>, x, x.start, x.end);
+r#impl!(RangeFrom<usize>, x, x.start);
+r#impl!(RangeFull);
+// [usize::MAX] will always be a valid inclusive end index.
+#[rustfmt::skip]
+r#impl!(RangeInclusive<usize>, x, *x.start(), x.end().wrapping_add(1));
+r#impl!(RangeTo<usize>, x, x.end);
+r#impl!(RangeToInclusive<usize>, x, x.end.wrapping_add(1));
+
 #[cfg(feature = "uniquote")]
 #[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "uniquote")))]
 impl Quote for RawOsStr {
@@ -684,12 +837,10 @@ impl ToOwned for RawOsStr {
 
 /// A container for the byte strings converted by [`OsStringBytes`].
 ///
-/// This wrapper is intended to prevent violating the invariants of the
-/// [unspecified encoding] used by this crate and minimize encoding
-/// conversions.
+/// For more information, see [`RawOsStr`].
 ///
 /// [unspecified encoding]: super#encoding
-#[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+#[derive(Clone, Debug, Default, Eq, Hash, Ord, PartialEq, PartialOrd)]
 #[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "raw_os_str")))]
 pub struct RawOsString(Vec<u8>);
 
@@ -824,7 +975,7 @@ impl Deref for RawOsString {
 
     #[inline]
     fn deref(&self) -> &Self::Target {
-        unsafe { RawOsStr::from_raw_bytes_unchecked(&self.0) }
+        RawOsStr::from_raw_bytes_unchecked(&self.0)
     }
 }
 
@@ -842,6 +993,25 @@ impl From<RawOsString> for Cow<'_, RawOsStr> {
     }
 }
 
+macro_rules! r#impl {
+    ( $index_type:ty ) => {
+        impl Index<$index_type> for RawOsString {
+            type Output = RawOsStr;
+
+            #[inline]
+            fn index(&self, idx: $index_type) -> &Self::Output {
+                &(**self)[idx]
+            }
+        }
+    };
+}
+r#impl!(Range<usize>);
+r#impl!(RangeFrom<usize>);
+r#impl!(RangeFull);
+r#impl!(RangeInclusive<usize>);
+r#impl!(RangeTo<usize>);
+r#impl!(RangeToInclusive<usize>);
+
 #[cfg(feature = "uniquote")]
 #[cfg_attr(os_str_bytes_docs_rs, doc(cfg(feature = "uniquote")))]
 impl Quote for RawOsString {
